@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import warnings
+
 import six
 import toolz
 
@@ -32,6 +34,7 @@ from ibis.expr.types import (Expr,  # noqa
                              StringValue, StringScalar, StringColumn,
                              DecimalValue, DecimalScalar, DecimalColumn,
                              TimestampValue, TimestampScalar, TimestampColumn,
+                             DateValue,
                              ArrayValue, ArrayScalar, ArrayColumn,
                              CategoryValue, unnamed, as_value_expr, literal,
                              null, sequence)
@@ -41,7 +44,6 @@ from ibis.expr.temporal import *  # noqa
 
 import ibis.common as _com
 
-from ibis.compat import py_string
 from ibis.expr.analytics import bucket, histogram
 from ibis.expr.groupby import GroupedTableExpr  # noqa
 from ibis.expr.window import window, trailing_window, cumulative_window
@@ -149,11 +151,16 @@ def timestamp(value):
     """
     Returns a timestamp literal if value is likely coercible to a timestamp
     """
-    if isinstance(value, py_string):
+    if isinstance(value, six.string_types):
         from pandas import Timestamp
         value = Timestamp(value)
-    op = ir.Literal(value)
-    return ir.TimestampScalar(op)
+    if isinstance(value, six.integer_types):
+        warnings.warn(
+            'Integer values for timestamp literals are deprecated in 0.11.0 '
+            'and will be removed in 0.12.0. To pass integers as timestamp '
+            'literals, use pd.Timestamp({:d}, unit=...)'.format(value)
+        )
+    return ir.TimestampScalar(ir.literal(value).op())
 
 
 schema.__doc__ = """\
@@ -767,41 +774,6 @@ _generic_value_methods = dict(
     cases=cases,
     substitute=substitute,
 
-    __add__=add,
-    add=add,
-
-    __sub__=sub,
-    sub=sub,
-
-    __mul__=mul,
-    mul=mul,
-
-    __div__=div,
-    __truediv__=div,
-    __floordiv__=floordiv,
-    div=div,
-    floordiv=floordiv,
-
-    __rdiv__=rdiv,
-    __rtruediv__=rdiv,
-    __rfloordiv__=rfloordiv,
-    rdiv=rdiv,
-    rfloordiv=rfloordiv,
-
-    __pow__=pow,
-    pow=pow,
-
-    __radd__=add,
-
-    __rsub__=rsub,
-    rsub=rsub,
-
-    __rmul__=_rbinop_expr('__rmul__', _ops.Multiply),
-    __rpow__=_binop_expr('__rpow__', _ops.Power),
-
-    __mod__=mod,
-    __rmod__=_rbinop_expr('__rmod__', _ops.Modulus),
-
     __eq__=_binop_expr('__eq__', _ops.Equals),
     __ne__=_binop_expr('__ne__', _ops.NotEquals),
     __ge__=_binop_expr('__ge__', _ops.GreaterEqual),
@@ -1080,6 +1052,41 @@ _numeric_value_methods = dict(
     round=round,
     nullifzero=_unary_op('nullifzero', _ops.NullIfZero),
     zeroifnull=_unary_op('zeroifnull', _ops.ZeroIfNull),
+
+    __add__=add,
+    add=add,
+
+    __sub__=sub,
+    sub=sub,
+
+    __mul__=mul,
+    mul=mul,
+
+    __div__=div,
+    __truediv__=div,
+    __floordiv__=floordiv,
+    div=div,
+    floordiv=floordiv,
+
+    __rdiv__=rdiv,
+    __rtruediv__=rdiv,
+    __rfloordiv__=rfloordiv,
+    rdiv=rdiv,
+    rfloordiv=rfloordiv,
+
+    __pow__=pow,
+    pow=pow,
+
+    __radd__=add,
+
+    __rsub__=rsub,
+    rsub=rsub,
+
+    __rmul__=_rbinop_expr('__rmul__', _ops.Multiply),
+    __rpow__=_binop_expr('__rpow__', _ops.Power),
+
+    __mod__=mod,
+    __rmod__=_rbinop_expr('__rmod__', _ops.Modulus),
 )
 
 
@@ -1602,6 +1609,9 @@ _string_value_methods = dict(
     join=_string_join,
     lpad=_lpad,
     rpad=_rpad,
+    __add__=add,
+    __mul__=mul,
+    __rmul__=mul,
 )
 
 
@@ -1703,7 +1713,16 @@ _timestamp_value_methods = dict(
 )
 
 
+_date_value_methods = dict(
+    strftime=_timestamp_strftime,
+    year=_extract_field('year', _ops.ExtractYear),
+    month=_extract_field('month', _ops.ExtractMonth),
+    day=_extract_field('day', _ops.ExtractDay),
+)
+
+
 _add_methods(TimestampValue, _timestamp_value_methods)
+_add_methods(DateValue, _date_value_methods)
 
 
 # ---------------------------------------------------------------------
@@ -1767,35 +1786,79 @@ def join(left, right, predicates=(), how='inner'):
     """
     klass = _join_classes[how.lower()]
     if isinstance(predicates, Expr):
-        predicates = _L.unwrap_ands(predicates)
+        predicates = _L.flatten_predicate(predicates)
 
     op = klass(left, right, predicates)
     return TableExpr(op)
 
 
-def cross_join(*args, **kwargs):
+def cross_join(*tables, **kwargs):
     """
     Perform a cross join (cartesian product) amongst a list of tables, with
     optional set of prefixes to apply to overlapping column names
 
     Parameters
     ----------
-    positional args: tables to join
-    prefixes keyword : prefixes for each table
-      Not yet implemented
-
-    Examples
-    --------
-    >>> joined1 = ibis.cross_join(a, b, c, d, e)
-    >>> joined2 = ibis.cross_join(a, b, c, prefixes=['a_', 'b_', 'c_']))
+    tables : ibis.expr.types.TableExpr
 
     Returns
     -------
     joined : TableExpr
-      If prefixes not provided, the result schema is not yet materialized
+
+    Examples
+    --------
+    >>> import ibis
+    >>> schemas = [(name, 'int64') for name in 'abcde']
+    >>> a, b, c, d, e = [
+    ...     ibis.table([(name, type)], name=name) for name, type in schemas
+    ... ]
+    >>> joined1 = ibis.cross_join(a, b, c, d, e)
+    >>> joined1  # doctest: +NORMALIZE_WHITESPACE
+    ref_0
+    UnboundTable[table]
+      name: a
+      schema:
+        a : int64
+    ref_1
+    UnboundTable[table]
+      name: b
+      schema:
+        b : int64
+    ref_2
+    UnboundTable[table]
+      name: c
+      schema:
+        c : int64
+    ref_3
+    UnboundTable[table]
+      name: d
+      schema:
+        d : int64
+    ref_4
+    UnboundTable[table]
+      name: e
+      schema:
+        e : int64
+    CrossJoin[table]
+      left:
+        Table: ref_0
+      right:
+        CrossJoin[table]
+          left:
+            CrossJoin[table]
+              left:
+                CrossJoin[table]
+                  left:
+                    Table: ref_1
+                  right:
+                    Table: ref_2
+              right:
+                Table: ref_3
+          right:
+            Table: ref_4
     """
-    op = _ops.CrossJoin(*args, **kwargs)
-    return TableExpr(op)
+    # TODO(phillipc): Implement prefix keyword argument
+    return TableExpr(_ops.CrossJoin(*tables, **kwargs))
 
 
 def _table_count(self):
@@ -1902,7 +1965,7 @@ def filter(table, predicates):
 
 def _resolve_predicates(table, predicates):
     if isinstance(predicates, Expr):
-        predicates = _L.unwrap_ands(predicates)
+        predicates = _L.flatten_predicate(predicates)
     predicates = util.promote_list(predicates)
     predicates = [ir.bind_expr(table, x) for x in predicates]
     resolved_predicates = []
