@@ -13,26 +13,41 @@
 # limitations under the License.
 
 import re
+import datetime
 
 from collections import namedtuple, OrderedDict
 
 import six
 
+import numpy as np
+
+import ibis
 import ibis.common as com
 import ibis.util as util
 
+from ibis.compat import builtins
+
 
 class Schema(object):
 
-    """
-    Holds table schema information
+    """An object for holding table schema information, i.e., column names and
+    types.
+
+    Parameters
+    ----------
+    names : Sequence[str]
+        A sequence of ``str`` indicating the name of each column.
+    types : Sequence[DataType]
+        A sequence of :class:`ibis.expr.datatypes.DataType` objects
+        representing type of each column.
     """
 
     def __init__(self, names, types):
         if not isinstance(names, list):
             names = list(names)
+
         self.names = names
-        self.types = [validate_type(x) for x in types]
+        self.types = [validate_type(typ) for typ in types]
 
         self._name_locs = dict((v, i) for i, v in enumerate(self.names))
 
@@ -90,9 +105,7 @@ class Schema(object):
 
     @classmethod
     def from_dict(cls, values):
-        names = list(values.keys())
-        types = values.values()
-        return Schema(names, types)
+        return Schema(list(values.keys()), values.values())
 
     def equals(self, other, cache=None):
         return self.names == other.names and self.types == other.types
@@ -111,6 +124,18 @@ class Schema(object):
     def items(self):
         return zip(self.names, self.types)
 
+    def name_at_position(self, i):
+        """
+        """
+        upper = len(self.names) - 1
+        if not 0 <= i <= upper:
+            raise ValueError(
+                'Column index must be between 0 and {:d}, inclusive'.format(
+                    upper
+                )
+            )
+        return self.names[i]
+
 
 class HasSchema(object):
 
@@ -190,8 +215,10 @@ class DataType(object):
         if isinstance(other, six.string_types):
             other = validate_type(other)
 
-        return (isinstance(other, type(self)) and
-                self.nullable == other.nullable)
+        return (
+            isinstance(other, type(self)) and
+            self.nullable == other.nullable
+        ) or isinstance(self, Any) or isinstance(other, Any)
 
     def can_implicit_cast(self, other):
         return self.equals(other)
@@ -204,9 +231,18 @@ class DataType(object):
         import ibis.expr.types as ir
         return getattr(ir, '{0}Column'.format(type(self).__name__))
 
+    def valid_literal(self, value):
+        raise NotImplementedError(
+            'valid_literal not implemented for datatype {}'.format(
+                type(self).__name__
+            )
+        )
+
 
 class Any(DataType):
-    pass
+
+    def valid_literal(self, value):
+        return True
 
 
 class Primitive(DataType):
@@ -214,7 +250,9 @@ class Primitive(DataType):
 
 
 class Null(DataType):
-    pass
+
+    def valid_literal(self, value):
+        return value is None or value is ibis.null
 
 
 class Variadic(DataType):
@@ -222,10 +260,15 @@ class Variadic(DataType):
 
 
 class Boolean(Primitive):
-    pass
+
+    def valid_literal(self, value):
+        return isinstance(value, bool) or (
+            isinstance(value, six.integer_types + (np.integer,)) and
+            (value == 0 or value == 1)
+        )
 
 
-Bounds = namedtuple('Bounds', ('upper', 'lower'))
+Bounds = namedtuple('Bounds', ('lower', 'upper'))
 
 
 class Integer(Primitive):
@@ -242,17 +285,74 @@ class Integer(Primitive):
             (type(self) is Integer or other._nbytes <= self._nbytes)
         )
 
+    def valid_literal(self, value):
+        lower, upper = self.bounds
+        return isinstance(
+            value, six.integer_types + (np.integer,)
+        ) and lower <= value <= upper
+
 
 class String(Variadic):
-    pass
+
+    def valid_literal(self, value):
+        return isinstance(value, six.string_types)
 
 
 class Date(Primitive):
-    pass
+
+    def valid_literal(self, value):
+        return isinstance(value, six.string_types + (datetime.date,))
 
 
+def parametric(cls):
+    type_name = cls.__name__
+    array_type_name = '{0}Column'.format(type_name)
+    scalar_type_name = '{0}Scalar'.format(type_name)
+
+    def array_type(self):
+        def constructor(op, name=None):
+            import ibis.expr.types as ir
+            return getattr(ir, array_type_name)(op, self, name=name)
+        return constructor
+
+    def scalar_type(self):
+        def constructor(op, name=None):
+            import ibis.expr.types as ir
+            return getattr(ir, scalar_type_name)(op, self, name=name)
+        return constructor
+
+    cls.array_type = array_type
+    cls.scalar_type = scalar_type
+    return cls
+
+
+@parametric
 class Timestamp(Primitive):
-    pass
+
+    def __init__(self, timezone=None, nullable=True):
+        super(Timestamp, self).__init__(nullable=nullable)
+        self.timezone = timezone
+
+    def equals(self, other, cache=None):
+        return super(Timestamp, self).equals(other, cache=cache) and (
+            self.timezone == validate_type(other).timezone
+        )
+
+    def __call__(self, timezone=None, nullable=True):
+        return type(self)(timezone=timezone, nullable=nullable)
+
+    def __str__(self):
+        timezone = self.timezone
+        typename = '{0.__class__.__name__}'.format(self).lower()
+        if timezone is None:
+            return typename
+        return '{}({!r})'.format(typename, timezone)
+
+    def __repr__(self):
+        return '{0.__class__.__name__}(timezone={0.timezone!r})'.format(self)
+
+    def valid_literal(self, value):
+        return isinstance(value, six.string_types + (datetime.datetime,))
 
 
 class SignedInteger(Integer):
@@ -270,6 +370,12 @@ class Floating(Primitive):
         else:
             return False
 
+    def valid_literal(self, value):
+        valid_floating_types = (
+            builtins.float, np.floating, np.integer
+        ) + six.integer_types
+        return isinstance(value, valid_floating_types)
+
 
 class Int8(Integer):
 
@@ -301,28 +407,6 @@ class Double(Floating):
     _nbytes = 8
 
 
-def parametric(cls):
-    type_name = cls.__name__
-    array_type_name = '{0}Column'.format(type_name)
-    scalar_type_name = '{0}Scalar'.format(type_name)
-
-    def array_type(self):
-        def constructor(op, name=None):
-            import ibis.expr.types as ir
-            return getattr(ir, array_type_name)(op, self, name=name)
-        return constructor
-
-    def scalar_type(self):
-        def constructor(op, name=None):
-            import ibis.expr.types as ir
-            return getattr(ir, scalar_type_name)(op, self, name=name)
-        return constructor
-
-    cls.array_type = array_type
-    cls.scalar_type = scalar_type
-    return cls
-
-
 @parametric
 class Decimal(DataType):
     # Decimal types are parametric, we store the parameters in this object
@@ -407,8 +491,8 @@ class Struct(DataType):
 
     def __init__(self, names, types, nullable=True):
         super(Struct, self).__init__(nullable=nullable)
-        self.names = names
-        self.types = types
+        self.names = list(names)
+        self.types = list(map(validate_type, types))
 
     def __repr__(self):
         return '{0}({1})'.format(
@@ -424,10 +508,11 @@ class Struct(DataType):
             )
         )
 
-    def __eq__(self, other):
-        return (isinstance(other, type(self)) and
-                self.names == other.names and
-                self.types == other.types)
+    def equals(self, other, cache=None):
+        return (
+            super(Struct, self).equals(other, cache=cache) and
+            self.names == other.names and self.types == other.types
+        )
 
     @classmethod
     def from_tuples(self, pairs):
@@ -439,7 +524,7 @@ class Array(Variadic):
 
     def __init__(self, value_type, nullable=True):
         super(Array, self).__init__(nullable=nullable)
-        self.value_type = value_type
+        self.value_type = validate_type(value_type)
 
     def __repr__(self):
         return '{0}({1})'.format(self.name, repr(self.value_type))
@@ -447,20 +532,30 @@ class Array(Variadic):
     def __str__(self):
         return '{0}<{1}>'.format(self.name.lower(), self.value_type)
 
-    def __eq__(self, other):
+    def equals(self, other, cache=None):
         return (
-            isinstance(other, type(self)) and
-            self.value_type == other.value_type
+            super(Array, self).equals(other, cache=cache) and
+            self.value_type.equals(other.value_type, cache=cache)
         )
 
+    def valid_literal(self, value):
+        return isinstance(value, (list, tuple))
+
 
 @parametric
 class Enum(DataType):
 
     def __init__(self, rep_type, value_type, nullable=True):
         super(Enum, self).__init__(nullable=nullable)
-        self.rep_type = rep_type
-        self.value_type = value_type
+        self.rep_type = validate_type(rep_type)
+        self.value_type = validate_type(value_type)
+
+    def equals(self, other, cache=None):
+        return (
+            super(Enum, self).equals(other, cache=cache) and
+            self.rep_type.equals(other.rep_type, cache=cache) and
+            self.value_type.equals(other.value_type, cache=cache)
+        )
 
 
 @parametric
@@ -468,8 +563,8 @@ class Map(DataType):
 
     def __init__(self, key_type, value_type, nullable=True):
         super(Map, self).__init__(nullable=nullable)
-        self.key_type = key_type
-        self.value_type = value_type
+        self.key_type = validate_type(key_type)
+        self.value_type = validate_type(value_type)
 
     def __repr__(self):
         return '{0}({1}, {2})'.format(
@@ -485,11 +580,11 @@ class Map(DataType):
             self.value_type,
         )
 
-    def __eq__(self, other):
+    def equals(self, other, cache=None):
         return (
-            isinstance(other, type(self)) and
-            self.key_type == other.key_type and
-            self.value_type == other.value_type
+            super(Map, self).equals(other, cache=cache) and
+            self.key_type.equals(other.key_type, cache=cache) and
+            self.value_type.equals(other.value_type, cache=cache)
         )
 
 
@@ -548,6 +643,8 @@ class Tokens(object):
     RPAREN = 14
     LBRACKET = 15
     RBRACKET = 16
+    TIMEZONE = 17
+    TIMESTAMP = 18
 
     @staticmethod
     def name(value):
@@ -563,6 +660,10 @@ _token_names = dict(
 Token = namedtuple('Token', ('type', 'value'))
 
 
+# Adapted from tokenize.String
+_STRING_REGEX = """('[^\n'\\\\]*(?:\\\\.[^\n'\\\\]*)*'|"[^\n"\\\\"]*(?:\\\\.[^\n"\\\\]*)*")"""  # noqa: E501
+
+
 _TYPE_RULES = OrderedDict(
     [
         # any, null
@@ -574,7 +675,13 @@ _TYPE_RULES = OrderedDict(
             '(?P<{}>{})'.format(token.upper(), token),
             lambda token, value=value: Token(Tokens.PRIMITIVE, value)
         ) for token, value in _primitive_types.items()
-        if token != 'any' and token != 'null'
+        if token not in {'any', 'null', 'timestamp'}
+    ] + [
+        # timestamp
+        (
+            r'(?P<TIMESTAMP>timestamp)',
+            lambda token: Token(Tokens.TIMESTAMP, token),
+        ),
     ] + [
         # decimal + complex types
         (
@@ -592,7 +699,7 @@ _TYPE_RULES = OrderedDict(
             ),
         )
     ] + [
-        # numbers, for decimal spec
+        # integers, for decimal spec
         (r'(?P<INTEGER>\d+)', lambda token: Token(Tokens.INTEGER, int(token))),
 
         # struct fields
@@ -600,6 +707,7 @@ _TYPE_RULES = OrderedDict(
             r'(?P<FIELD>[a-zA-Z_][a-zA-Z_0-9]*)',
             lambda token: Token(Tokens.FIELD, token)
         ),
+        # timezones
         ('(?P<COMMA>,)', lambda token: Token(Tokens.COMMA, token)),
         ('(?P<COLON>:)', lambda token: Token(Tokens.COLON, token)),
         (r'(?P<LPAREN>\()', lambda token: Token(Tokens.LPAREN, token)),
@@ -607,6 +715,10 @@ _TYPE_RULES = OrderedDict(
         ('(?P<LBRACKET><)', lambda token: Token(Tokens.LBRACKET, token)),
         ('(?P<RBRACKET>>)', lambda token: Token(Tokens.RBRACKET, token)),
         (r'(?P<WHITESPACE>\s+)', None),
+        (
+            '(?P<TIMEZONE>{})'.format(_STRING_REGEX),
+            lambda token: Token(Tokens.TIMEZONE, token),
+        ),
     ]
 )
 
@@ -708,7 +820,10 @@ class TypeParser(object):
                   | "float"
                   | "double"
                   | "string"
-                  | "timestamp"
+                  | timestamp
+
+        timestamp : "timestamp"
+                  | "timestamp" "(" timezone ")"
 
         decimal : "decimal"
                 | "decimal" "(" integer "," integer ")"
@@ -726,6 +841,14 @@ class TypeParser(object):
         if self._accept(Tokens.PRIMITIVE):
             return self.tok.value
 
+        elif self._accept(Tokens.TIMESTAMP):
+            if self._accept(Tokens.LPAREN):
+                self._expect(Tokens.TIMEZONE)
+                timezone = self.tok.value[1:-1]  # remove surrounding quotes
+                self._expect(Tokens.RPAREN)
+                return Timestamp(timezone=timezone)
+            return timestamp
+
         elif self._accept(Tokens.DECIMAL):
             if self._accept(Tokens.LPAREN):
 
@@ -794,7 +917,7 @@ class TypeParser(object):
             self._expect(Tokens.RBRACKET)
             return Struct(names, types)
         else:
-            raise SyntaxError('Type cannot be parsed: {0}'.format(self.text))
+            raise SyntaxError('Type cannot be parsed: {}'.format(self.text))
 
 
 def validate_type(t):
