@@ -75,6 +75,7 @@ class GroupedTableExpr(object):
 
         Parameters
         ----------
+        expr : ibis.expr.types.Expr
 
         Returns
         -------
@@ -82,9 +83,10 @@ class GroupedTableExpr(object):
         """
         exprs = util.promote_list(expr)
         new_having = self._having + exprs
-        return GroupedTableExpr(self.table, self.by, having=new_having,
-                                order_by=self._order_by,
-                                window=self._window)
+        return GroupedTableExpr(
+            self.table, self.by,
+            having=new_having, order_by=self._order_by, window=self._window
+        )
 
     def order_by(self, expr):
         """
@@ -101,9 +103,10 @@ class GroupedTableExpr(object):
         """
         exprs = util.promote_list(expr)
         new_order = self._order_by + exprs
-        return GroupedTableExpr(self.table, self.by, having=self._having,
-                                order_by=new_order,
-                                window=self._window)
+        return GroupedTableExpr(
+            self.table, self.by,
+            having=self._having, order_by=new_order, window=self._window
+        )
 
     def mutate(self, exprs=None, **kwds):
         """
@@ -117,11 +120,54 @@ class GroupedTableExpr(object):
 
         Examples
         --------
-        >>> expr = (table
-                    .group_by('foo')
-                    .order_by(ibis.desc('bar'))
-                    .mutate(qux=lambda x: x.baz.lag(),
-                            qux2=table.baz.lead()))
+        >>> import ibis
+        >>> t = ibis.table([
+        ...     ('foo', 'string'),
+        ...     ('bar', 'string'),
+        ...     ('baz', 'double'),
+        ... ], name='t')
+        >>> t
+        UnboundTable[table]
+          name: t
+          schema:
+            foo : string
+            bar : string
+            baz : double
+        >>> expr = (t.group_by('foo')
+        ...          .order_by(ibis.desc('bar'))
+        ...          .mutate(qux=lambda x: x.baz.lag(),
+        ...                  qux2=t.baz.lead()))
+        >>> print(expr)  # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+        ref_0
+        UnboundTable[table]
+          name: t
+          schema:
+            foo : string
+            bar : string
+            baz : double
+        Selection[table]
+          table:
+            Table: ref_0
+          selections:
+            Table: ref_0
+            qux = WindowOp[double*]
+              qux = Lag[double*]
+                baz = Column[double*] 'baz' from table
+                  ref_0
+                offset:
+                  None
+                default:
+                  None
+              <ibis.expr.window.Window object at 0x...>
+            qux2 = WindowOp[double*]
+              qux2 = Lead[double*]
+                baz = Column[double*] 'baz' from table
+                  ref_0
+                offset:
+                  None
+                default:
+                  None
+              <ibis.expr.window.Window object at 0x...>
 
         Returns
         -------
