@@ -71,6 +71,44 @@ def test_literal_cases(value, expected_type):
     assert expr.op().value is value
 
 
+@pytest.mark.parametrize(
+    ['value', 'expected_type'],
+    [
+        (5, 'int16'),
+        (127, 'double'),
+        (128, 'int64'),
+        (32767, 'double'),
+        (32768, 'float'),
+        (2147483647, 'int64'),
+        (-5, 'int16'),
+        (-128, 'int32'),
+        (-129, 'int64'),
+        (-32769, 'float'),
+        (-2147483649, 'double'),
+        (1.5, 'double'),
+        ('foo', 'string'),
+    ]
+)
+def test_literal_with_different_type(value, expected_type):
+    expr = ibis.literal(value, type=expected_type)
+    assert expr.type().equals(dt.validate_type(expected_type))
+
+
+@pytest.mark.parametrize(
+    ['value', 'expected_type'],
+    [
+        (32767, 'int8'),
+        (32768, 'int16'),
+        (2147483647, 'int16'),
+        (2147483648, 'int32'),
+        ('foo', 'double'),
+    ]
+)
+def test_literal_with_different_type_failure(value, expected_type):
+    with pytest.raises(TypeError):
+        ibis.literal(value, type=expected_type)
+
+
 def test_literal_list():
     what = [1, 2, 1000]
     expr = api.as_value_expr(what)
@@ -756,3 +794,66 @@ def test_not_without_boolean(typ):
     c = t.a
     with pytest.raises(TypeError):
         ~c
+
+
+@pytest.mark.parametrize(
+    ('position', 'names'),
+    [
+        (0, 'foo'),
+        (1, 'bar'),
+        ([0], ['foo']),
+        ([1], ['bar']),
+        ([0, 1], ['foo', 'bar']),
+        ([1, 0], ['bar', 'foo']),
+    ]
+)
+@pytest.mark.parametrize(
+    'expr_func',
+    [
+        lambda t, args: t[args],
+        lambda t, args: t.sort_by(args),
+        lambda t, args: t.group_by(args).aggregate(bar_avg=t.bar.mean())
+    ]
+)
+def test_table_operations_with_integer_column(position, names, expr_func):
+    t = ibis.table([('foo', 'string'), ('bar', 'double')])
+    result = expr_func(t, position)
+    expected = expr_func(t, names)
+    assert result.equals(expected)
+
+
+@pytest.mark.parametrize(
+    'value',
+    [
+        'abcdefg',
+        ['a', 'b', 'c'],
+        [1, 2, 3],
+    ]
+)
+@pytest.mark.parametrize(
+    'operation',
+    [
+        'pow',
+        'sub',
+        'truediv',
+        'floordiv',
+        'mod',
+    ]
+)
+def test_generic_value_api_no_arithmetic(value, operation):
+    func = getattr(operator, operation)
+    expr = ibis.literal(value)
+    with pytest.raises(TypeError):
+        func(expr, expr)
+
+
+@pytest.mark.parametrize(
+    ('value', 'expected'),
+    [
+        (5, dt.int8),
+        (5.4, dt.double),
+        ('abc', dt.string),
+    ]
+)
+def test_fillna_null(value, expected):
+    assert ibis.NA.fillna(value).type().equals(expected)
