@@ -289,7 +289,7 @@ class ImpalaQuery(Query):
         names = [x[0] for x in cursor.description]
         return _column_batches_to_dataframe(names, batches)
 
-    def _db_type_to_dtype(self, db_type):
+    def _db_type_to_dtype(self, db_type, column):
         return _HS2_TTypeId_to_dtype[db_type]
 
 
@@ -840,18 +840,12 @@ class ImpalaClient(SQLClient):
             ast = self._build_ast(to_insert)
             select = ast.queries[0]
 
-            if partition is not None:
-                # Fairly certain this is currently the case
-                raise ValueError('partition not supported with '
-                                 'create-table-as-select. Create an '
-                                 'empty partitioned table instead '
-                                 'and insert into those partitions.')
-
             statement = ddl.CTAS(table_name, select,
                                  database=database,
                                  can_exist=force,
                                  format=format,
                                  external=external,
+                                 partition=partition,
                                  path=location)
         elif schema is not None:
             statement = ddl.CreateTableWithSchema(
