@@ -374,24 +374,45 @@ class _TableSetFormatter(comp.TableSetFormatter):
 
 class ImpalaUnion(comp.Union):
 
-    def compile(self):
+    def _extract_subqueries(self):
+        self.subqueries = comp._extract_subqueries(self)
+        for subquery in self.subqueries:
+            self.context.set_extracted(subquery)
+
+    def format_subqueries(self):
         context = self.context
+        subqueries = self.subqueries
 
-        if self.distinct:
-            union_keyword = 'UNION'
-        else:
-            union_keyword = 'UNION ALL'
+        return ',\n'.join([
+            '{0} AS (\n{1}\n)'.format(
+                context.get_ref(expr),
+                util.indent(context.get_compiled_expr(expr), 2)
+            ) for expr in subqueries
+        ])
 
-        left_set = context.get_compiled_expr(self.left, isolated=True)
-        right_set = context.get_compiled_expr(self.right, isolated=True)
+    def format_relation(self, expr):
+        ref = self.context.get_ref(expr)
+        if ref is not None:
+            return 'SELECT *\nFROM {0}'.format(ref)
+        return self.context.get_compiled_expr(expr)
 
-        # XXX: hack of all trades - our right relation has a CTE
-        # TODO: factor out common subqueries in the union
-        if right_set.startswith('WITH'):
-            format_string = '({0})\n{1}\n({2})'
-        else:
-            format_string = '{0}\n{1}\n{2}'
-        return format_string.format(left_set, union_keyword, right_set)
+    def compile(self):
+        union_keyword = 'UNION' if self.distinct else 'UNION ALL'
+
+        self._extract_subqueries()
+
+        left_set = self.format_relation(self.left)
+        right_set = self.format_relation(self.right)
+        extracted = self.format_subqueries()
+
+        buf = []
+
+        if extracted:
+            buf.append('WITH {0}'.format(extracted))
+
+        buf.extend([left_set, union_keyword, right_set])
+
+        return '\n'.join(buf)
 
 
 # ---------------------------------------------------------------------
@@ -485,7 +506,9 @@ def _window(translator, expr):
                          ops.DenseRank,
                          ops.MinRank,
                          ops.FirstValue,
-                         ops.LastValue)
+                         ops.LastValue,
+                         ops.PercentRank,
+                         ops.NTile,)
 
     _unsupported_reductions = (
         ops.CMSMedian,
@@ -617,6 +640,12 @@ def _nth_value(translator, expr):
                                                rank_formatted)
 
 
+def _ntile(translator, expr):
+    op = expr.op()
+    arg, buckets = map(translator.translate, op.args)
+    return 'ntile({})'.format(buckets)
+
+
 def _negate(translator, expr):
     arg = expr.op().args[0]
     formatted_arg = translator.translate(arg)
@@ -628,6 +657,10 @@ def _negate(translator, expr):
         return '-{0!s}'.format(formatted_arg)
 
 
+def _not(translator, expr):
+    return 'NOT {}'.format(*map(translator.translate, expr.op().args))
+
+
 def _parenthesize(what):
     return '({0!s})'.format(what)
 
@@ -1141,6 +1174,17 @@ def _value_list(translator, expr):
     return '({0})'.format(', '.join(formatted))
 
 
+def _identical_to(translator, expr):
+    op = expr.op()
+    if op.args[0].equals(op.args[1]):
+        return 'TRUE'
+
+    left, right = map(translator.translate, op.args)
+    return '{left} IS NOT DISTINCT FROM {right}'.format(
+        left=left, right=right
+    )
+
+
 _subtract_one = '({0} - 1)'.format
 
 
@@ -1148,6 +1192,7 @@ _expr_transforms = {
     ops.RowNumber: _subtract_one,
     ops.DenseRank: _subtract_one,
     ops.MinRank: _subtract_one,
+    ops.NTile: _subtract_one,
 }
 
 
@@ -1167,6 +1212,7 @@ _binary_infix_ops = {
     ops.Greater: _binary_infix_op('>'),
     ops.LessEqual: _binary_infix_op('<='),
     ops.Less: _binary_infix_op('<'),
+    ops.IdenticalTo: _identical_to,
 
     # Boolean comparisons
     ops.And: _binary_infix_op('AND'),
@@ -1180,6 +1226,7 @@ _operation_registry = {
     ops.NotNull: _not_null,
     ops.IsNull: _is_null,
     ops.Negate: _negate,
+    ops.Not: _not,
 
     ops.IfNull: _ifnull_workaround,
     ops.NullIf: fixed_arity('nullif', 2),
@@ -1296,13 +1343,15 @@ _operation_registry = {
     ops.RowNumber: lambda *args: 'row_number()',
     ops.DenseRank: lambda *args: 'dense_rank()',
     ops.MinRank: lambda *args: 'rank()',
+    ops.PercentRank: lambda *args: 'percent_rank()',
 
     ops.FirstValue: unary('first_value'),
     ops.LastValue: unary('last_value'),
     ops.NthValue: _nth_value,
     ops.Lag: _shift_like('lag'),
     ops.Lead: _shift_like('lead'),
-    ops.WindowOp: _window
+    ops.WindowOp: _window,
+    ops.NTile: _ntile,
 }
 
 _operation_registry.update(_binary_infix_ops)
@@ -1317,6 +1366,7 @@ class ImpalaExprTranslator(comp.ExprTranslator):
         return _name_expr(translated,
                           quote_identifier(name, force=force))
 
+
 compiles = ImpalaExprTranslator.compiles
 rewrites = ImpalaExprTranslator.rewrites
 
