@@ -12,28 +12,31 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# flake8: noqa=E402
+import unittest
 
 import pytest
 
-pytest.importorskip('sqlalchemy')
-pytest.importorskip('impala.dbapi')
-
 import pandas as pd
 import pandas.util.testing as tm
 
 import ibis
-
-from ibis import literal as L
-from ibis.compat import unittest, StringIO, Decimal
-from ibis.expr.datatypes import Category
-from ibis.expr.tests.mocks import MockConnection
-from ibis.impala.compiler import ImpalaExprTranslator, to_sql, ImpalaContext
-from ibis.sql.tests.test_compiler import ExprTestCases
-from ibis.impala.tests.common import ImpalaE2E
 import ibis.expr.types as ir
 import ibis.expr.api as api
 
+from ibis import literal as L
+from ibis.expr.datatypes import Category
+
+from ibis.compat import StringIO, Decimal
+from ibis.expr.tests.mocks import MockConnection
+
+pytest.importorskip('sqlalchemy')
+pytest.importorskip('impala.dbapi')
+
+from ibis.impala.compiler import ImpalaExprTranslator, to_sql  # noqa: E402
+from ibis.impala.compiler import ImpalaContext  # noqa: E402
+from ibis.sql.tests.test_compiler import ExprTestCases  # noqa: E402
+from ibis.impala.tests.common import ImpalaE2E  # noqa: E402
+
 
 def approx_equal(a, b, eps):
     assert abs(a - b) < eps
@@ -505,7 +508,6 @@ FROM functional_alltypes"""
         assert result == 'SELECT TRUE AS `tmp`'
 
 
-
 class TestBucketHistogram(unittest.TestCase, ExprSQLTest):
 
     def setUp(self):
@@ -1444,10 +1446,9 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
             d.var(where=cond),
         ]
 
-        agg_exprs = [expr.name('e%d' % i)
-                     for i, expr in enumerate(exprs)]
+        metrics = [expr.name('e%d' % i) for i, expr in enumerate(exprs)]
 
-        agged_table = table.aggregate(agg_exprs)
+        agged_table = table.aggregate(metrics)
         agged_table.execute()
 
     def test_analytic_functions(self):
