@@ -14,6 +14,9 @@
 
 import numbers
 import operator
+import functools
+import contextlib
+
 import six
 
 import sqlalchemy as sa
@@ -22,8 +25,14 @@ import sqlalchemy.sql as sql
 from sqlalchemy.sql.elements import Over as _Over
 from sqlalchemy.ext.compiler import compiles as sa_compiles
 
-from ibis.client import SQLClient, AsyncQuery, Query
+import numpy as np
+import pandas as pd
+
+from ibis.client import SQLClient, AsyncQuery, Query, Database
 from ibis.sql.compiler import Select, Union, TableSetFormatter
+
+from ibis import compat
+
 import ibis.common as com
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
@@ -50,9 +59,8 @@ _ibis_type_to_sqla = {
 
     dt.Date: sa.Date,
 
-    dt.Timestamp: sa.DateTime,
-
     dt.Decimal: sa.NUMERIC,
+    dt.Null: sa.types.NullType,
 }
 
 _sqla_type_mapping = {
@@ -80,7 +88,9 @@ _sqla_type_to_ibis = dict((v, k) for k, v in _ibis_type_to_sqla.items())
 _sqla_type_to_ibis.update(_sqla_type_mapping)
 
 
-def sqlalchemy_type_to_ibis_type(column_type, nullable=True):
+def sqlalchemy_type_to_ibis_type(
+    column_type, nullable=True, default_timezone=None
+):
     type_class = type(column_type)
 
     if isinstance(column_type, sa.types.NUMERIC):
@@ -91,14 +101,20 @@ def sqlalchemy_type_to_ibis_type(column_type, nullable=True):
         if type_class in _sqla_type_to_ibis:
             ibis_class = _sqla_type_to_ibis[type_class]
         elif isinstance(column_type, sa.DateTime):
-            ibis_class = dt.Timestamp()
+            return dt.Timestamp(
+                timezone=default_timezone if column_type.timezone else None,
+                nullable=nullable
+            )
         elif isinstance(column_type, sa.ARRAY):
             dimensions = column_type.dimensions
             if dimensions is not None and dimensions != 1:
                 raise NotImplementedError(
                     'Nested array types not yet supported'
                 )
-            value_type = sqlalchemy_type_to_ibis_type(column_type.item_type)
+            value_type = sqlalchemy_type_to_ibis_type(
+                column_type.item_type,
+                default_timezone=default_timezone,
+            )
 
             def make_array_type(nullable, value_type=value_type):
                 return dt.Array(value_type, nullable=nullable)
@@ -120,9 +136,24 @@ def sqlalchemy_type_to_ibis_type(column_type, nullable=True):
 
 
 def schema_from_table(table):
+    """Retrieve an ibis schema from a SQLAlchemy ``Table``.
+
+    Parameters
+    ----------
+    table : sa.Table
+
+    Returns
+    -------
+    schema : ibis.expr.datatypes.Schema
+        An ibis schema corresponding to the types of the columns in `table`.
+    """
     # Convert SQLA table to Ibis schema
     types = [
-        sqlalchemy_type_to_ibis_type(column.type, column.nullable)
+        sqlalchemy_type_to_ibis_type(
+            column.type,
+            nullable=column.nullable,
+            default_timezone='UTC',
+        )
         for column in table.columns.values()
     ]
     return dt.Schema(table.columns.keys(), types)
@@ -146,6 +177,10 @@ def _to_sqla_type(itype, type_map=None):
         type_map = _ibis_type_to_sqla
     if isinstance(itype, dt.Decimal):
         return sa.types.NUMERIC(itype.precision, itype.scale)
+    elif isinstance(itype, dt.Timestamp):
+        # SQLAlchemy DateTimes do not store the timezone, just whether the db
+        # supports timezones.
+        return sa.TIMESTAMP(bool(itype.timezone))
     elif isinstance(itype, dt.Array):
         ibis_type = itype.value_type
         if not isinstance(ibis_type, (dt.Primitive, dt.String)):
@@ -159,6 +194,30 @@ def _to_sqla_type(itype, type_map=None):
         return type_map[type(itype)]
 
 
+def _variance_reduction(func_name):
+    suffix = {
+        'sample': 'samp',
+        'pop': 'pop'
+    }
+
+    def variance_compiler(t, expr):
+        arg, where, how = expr.op().args
+
+        if arg.type().equals(dt.boolean):
+            arg = arg.cast('int32')
+
+        func = getattr(
+            sa.func,
+            '{}_{}'.format(func_name, suffix.get(how, 'samp'))
+        )
+
+        if where is not None:
+            arg = where.ifelse(arg, None)
+        return func(t.translate(arg))
+
+    return variance_compiler
+
+
 def fixed_arity(sa_func, arity):
     if isinstance(sa_func, six.string_types):
         sa_func = getattr(sa.func, sa_func)
@@ -509,17 +568,86 @@ def build_ast(expr, context=None, dialect=None):
     return builder.get_result()
 
 
+class AlchemyDatabaseSchema(Database):
+
+    def __init__(self, name, database):
+        """
+
+        Parameters
+        ----------
+        name : str
+        database : AlchemyDatabase
+        """
+        self.name = name
+        self.database = database
+        self.client = database.client
+
+    def __repr__(self):
+        return "Schema({!r})".format(self.name)
+
+    def drop(self, force=False):
+        """
+        Drop the schema
+
+        Parameters
+        ----------
+        force : boolean, default False
+          Drop any objects if they exist, and do not fail if the schema does
+          not exist.
+        """
+        raise NotImplementedError(
+            "Drop is not implemented yet for sqlalchemy schemas")
+
+    def table(self, name):
+        """
+        Return a table expression referencing a table in this schema
+
+        Returns
+        -------
+        table : TableExpr
+        """
+        qualified_name = self._qualify(name)
+        return self.database.table(qualified_name, self.name)
+
+    def list_tables(self, like=None):
+        return self.database.list_tables(
+            schema=self.name,
+            like=self._qualify_like(like))
+
+
+class AlchemyDatabase(Database):
+    """
+
+    Attributes
+    ----------
+    client : AlchemyClient
+
+    """
+    schema_class = AlchemyDatabaseSchema
+
+    def table(self, name, schema=None):
+        return self.client.table(name, schema=schema)
+
+    def list_tables(self, like=None, schema=None):
+        return self.client.list_tables(
+            schema=schema,
+            like=self._qualify_like(like),
+            database=self.name)
+
+    def schema(self, name):
+        return self.schema_class(name, self)
+
+
 class AlchemyTable(ops.DatabaseTable):
 
-    def __init__(self, table, source):
+    def __init__(self, table, source, schema=None):
+        super(AlchemyTable, self).__init__(
+            table.name,
+            schema or schema_from_table(table),
+            source,
+        )
         self.sqla_table = table
 
-        schema = schema_from_table(table)
-        name = table.name
-
-        ops.TableNode.__init__(self, [name, schema, source])
-        ops.HasSchema.__init__(self, schema, name=name)
-
 
 class AlchemyExprTranslator(comp.ExprTranslator):
 
@@ -545,13 +673,47 @@ compiles = AlchemyExprTranslator.compiles
 class AlchemyQuery(Query):
 
     def _fetch(self, cursor):
-        # No guarantees that the DBAPI cursor has data types
-        import pandas as pd
-        proxy = cursor.proxy
-        rows = proxy.fetchall()
-        colnames = proxy.keys()
-        return pd.DataFrame.from_records(rows, columns=colnames,
-                                         coerce_float=True)
+        df = pd.DataFrame.from_records(
+            cursor.proxy.fetchall(),
+            columns=cursor.proxy.keys(),
+            coerce_float=True
+        )
+        dtypes = df.dtypes
+        for column in df.columns:
+            existing_dtype = dtypes[column]
+            try:
+                db_type = _to_sqla_type(self.schema[column])
+            except KeyError:
+                new_dtype = existing_dtype
+            else:
+                try:
+                    new_dtype = self._db_type_to_dtype(db_type, column)
+                except TypeError:
+                    new_dtype = existing_dtype
+
+            if getattr(
+                existing_dtype, 'tz', None
+            ) != getattr(new_dtype, 'tz', None):
+                df[column] = df[column].astype(new_dtype)
+        return df
+
+    def _db_type_to_dtype(self, db_type, column):
+        if isinstance(db_type, sa.DateTime):
+            if not db_type.timezone:
+                return np.dtype('datetime64[ns]')
+            else:
+                return compat.DatetimeTZDtype(
+                    'ns', self.schema[column].timezone
+                )
+        raise TypeError(repr(db_type))
+
+    @property
+    def schema(self):
+        expr = self.expr
+        try:
+            return expr.schema()
+        except AttributeError:
+            return ibis.schema([(expr.get_name(), expr.type())])
 
 
 class AlchemyAsyncQuery(AsyncQuery):
@@ -563,15 +725,58 @@ class AlchemyDialect(object):
     translator = AlchemyExprTranslator
 
 
+def invalidates_reflection_cache(f):
+    """Invalidate the SQLAlchemy reflection cache if `f` performs an operation
+    that mutates database or table metadata such as ``CREATE TABLE``,
+    ``DROP TABLE``, etc.
+
+    Parameters
+    ----------
+    f : callable
+        A method on :class:`ibis.sql.alchemy.AlchemyClient`
+    """
+    @functools.wraps(f)
+    def wrapped(self, *args, **kwargs):
+        result = f(self, *args, **kwargs)
+
+        # only invalidate the cache after we've succesfully called the wrapped
+        # function
+        self._reflection_cache_is_dirty = True
+        return result
+    return wrapped
+
+
 class AlchemyClient(SQLClient):
 
     dialect = AlchemyDialect
     sync_query = AlchemyQuery
 
+    def __init__(self, con):
+        super(AlchemyClient, self).__init__()
+        self.con = con
+        self.meta = sa.MetaData(bind=con)
+        self._inspector = sa.inspect(con)
+        self._reflection_cache_is_dirty = False
+        self._schemas = {}
+
     @property
     def async_query(self):
-        raise NotImplementedError
+        raise NotImplementedError(
+            'async_query not implemented in {}'.format(type(self).__name__)
+        )
 
+    @property
+    def inspector(self):
+        if self._reflection_cache_is_dirty:
+            self._inspector.info_cache.clear()
+        return self._inspector
+
+    @contextlib.contextmanager
+    def begin(self):
+        with self.con.begin() as bind:
+            yield bind
+
+    @invalidates_reflection_cache
     def create_table(self, name, expr=None, schema=None, database=None):
         if database is not None and database != self.engine.url.database:
             raise NotImplementedError(
@@ -588,14 +793,20 @@ class AlchemyClient(SQLClient):
                     'Expression schema is not equal to passed schema. '
                     'Try passing the expression without the schema'
                 )
-        t = table_from_schema(name, self.meta, schema or expr.schema())
-        with self.con.begin() as bind:
+        if schema is None:
+            schema = expr.schema()
+
+        self._schemas[self._fully_qualified_name(name, database)] = schema
+        t = table_from_schema(name, self.meta, schema)
+
+        with self.begin() as bind:
             t.create(bind=bind)
             if expr is not None:
                 bind.execute(
                     t.insert().from_select(list(expr.columns), expr.compile())
                 )
 
+    @invalidates_reflection_cache
     def drop_table(self, table_name, database=None, force=False):
         if database is not None and database != self.engine.url.database:
             raise NotImplementedError(
@@ -610,12 +821,19 @@ class AlchemyClient(SQLClient):
             )
         self.meta.remove(t)
 
+        qualified_name = self._fully_qualified_name(table_name, database)
+
+        try:
+            del self._schemas[qualified_name]
+        except KeyError:  # schemas won't be cached if created with raw_sql
+            pass
+
     def truncate_table(self, table_name, database=None):
         self.meta.tables[table_name].delete().execute()
 
-    def list_tables(self, like=None, database=None):
+    def list_tables(self, like=None, database=None, schema=None):
         """
-        List tables in the current (or indicated) database.
+        List tables/views in the current (or indicated) database.
 
         Parameters
         ----------
@@ -628,21 +846,26 @@ class AlchemyClient(SQLClient):
         -------
         tables : list of strings
         """
-        if database is None:
-            database = self.current_database
-        names = self.con.table_names(schema=database)
+        inspector = self.inspector
+        names = inspector.get_table_names(schema=schema)
+        names.extend(inspector.get_view_names(schema=schema))
         if like is not None:
             names = [x for x in names if like in x]
-        return names
+        return sorted(names)
 
     def _execute(self, query, results=True):
-        return AlchemyProxy(self.con.execute(query))
+        with self.begin() as con:
+            return AlchemyProxy(con.execute(query))
+
+    @invalidates_reflection_cache
+    def raw_sql(self, query, results=False):
+        return super(AlchemyClient, self).raw_sql(query, results=results)
 
     def _build_ast(self, expr):
         return build_ast(expr, dialect=self.dialect)
 
-    def _get_sqla_table(self, name):
-        return sa.Table(name, self.meta, autoload=True)
+    def _get_sqla_table(self, name, schema=None):
+        return sa.Table(name, self.meta, schema=schema, autoload=True)
 
     def _sqla_table_to_expr(self, table):
         node = AlchemyTable(table, self)
@@ -653,7 +876,7 @@ class AlchemySelect(Select):
 
     def __init__(self, *args, **kwargs):
         self.exists = kwargs.pop('exists', False)
-        Select.__init__(self, *args, **kwargs)
+        super(AlchemySelect, self).__init__(*args, **kwargs)
 
     def compile(self):
         # Can't tell if this is a hack or not. Revisit later
